#!/usr/bin/perl -w 
############################## check_snmp_extreme_ap #################
my $Version='1.0';
# Date : Jan 21 2020
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use Encode qw(decode encode);

my $PROGNAME='check_snmp_extreme_ap.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=$ERRORS{OK};
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $type = 'cpu';
my $warning=80;
my $critical=90;

my $oid_sysDescr=".1.3.6.1.2.1.1.1";
my $oid_cpu="1.3.6.1.4.1.388.50.1.4.2.2.2.1.0";
my $oid_mem="1.3.6.1.4.1.388.50.1.4.2.2.2.3.0";
my $oid_client="1.3.6.1.4.1.388.50.1.4.2.25.1.1.1.4";
sub print_help() {
	printf "check SNMP Extreme Network Access Point.\n";
	printf "\tcheck cpu load, \n";
	printf "\tcheck memory\n";
	printf "\tcheck number of connetect client\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -T (--type)         test type (default:cpu) [cpu|mem|client]\n";
	print " -w                  warning threshold (default:70)\n";
	print " -c                  critical threshold (default:80)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
    "T=s" => \$type,"type=s" => \$type,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#sysDescr INFORMATION
my $resultat = $snmp->get_request("$oid_sysDescr.0");
if (!defined($resultat)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community,
		);
		$resultat = $snmp->get_request("$oid_sysDescr");
		if (!defined($resultat)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
   		$snmp->close;
  		 exit $ERRORS{"UNKNOWN"};
	}
}
my $description= $$resultat{"$oid_sysDescr.0"};

if($type  eq "cpu")
{
    my $load = getsnmpOID($snmp,"$oid_cpu");
    $load = ($load/10);
    $load = sprintf( "%.1f", $load);
    my $state = 'OK';
    my $message="< $warning%";
    if($load > $critical)
    {
        $state ="CRITICAL";
        $message="> $critical%";
    }
    elsif($load > $warning)
    {
        $state ="WARNING";
        $message="> $warning%";
    }
    print "$state : CPU is used $load% $message|cpu=$load\n";
    $return = priority_state($return,$ERRORS{$state});
}
elsif($type  eq "mem")
{
    my $memory = getsnmpOID($snmp,"$oid_mem");
    $memory = $memory/10;
    $memory = sprintf( "%.1f", $memory);
    $memory = 100 - $memory;
    my $state = "OK";
    my $message="< $warning%";
    if($memory > $critical)
    {
        $state ="CRITICAL";
        $message="> $critical%";
    }
    elsif($memory > $warning)
    {
        $state ="WARNING";
        $message="> $warning%";
    }
    print "$state : memory is used $memory% $message|mem=$memory\n";
    $return = priority_state($return,$ERRORS{$state});
}
elsif($type  eq "client")
{
    my $client =0;
    my $result = $snmp->get_table(Baseoid => $oid_client);
    foreach my $key ( keys %$result) 
    {
        my $value =$$result{"$key"};
        $client = $client + $value;
    }
    my $state = 'OK';
    my $message="< $warning";
    if($client > $critical)
    {
        $state ="CRITICAL";
        $message="> $critical";
    }
    elsif($client > $warning)
    {
        $state ="WARNING";
        $message="> $warning";
    }
    print "$state : The number of connected  client is $client $message|client=$client\n";
    $return = priority_state($return,$ERRORS{$state});
}
exit($return);

