#!/usr/bin/perl -w 
############################## check_snmp_hiKvision #################
my $Version='1.0';
# Date : Jan 21 2020
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use Encode qw(decode encode);

my $PROGNAME='check_snmp_hiKvision.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=$ERRORS{OK};
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 1;
my $type = 'status';
my $warning=80;
my $critical=90;

my $oid_sysDescr=".1.3.6.1.2.1.1.1";
my $oid_serial=".1.3.6.1.4.1.50001.1.3.0";
my $oid_type_device=".1.3.6.1.4.1.50001.1.100.0";
my $oid_is_online=".1.3.6.1.4.1.50001.1.102.0";
my $oid_device_state = ".1.3.6.1.4.1.50001.1.230.0";
my $oid_mem_usage = ".1.3.6.1.4.1.50001.1.221.0";
my $oid_disk_number = ".1.3.6.1.4.1.50001.1.240.0";
my $oid_disk_volume = ".1.3.6.1.4.1.50001.1.241.1.2";
my $oid_disk_status = ".1.3.6.1.4.1.50001.1.241.1.3";
my $oid_disk_size = "1.3.6.1.4.1.50001.1.241.1.5";
my %disk_status_text  = ( "0"  => "normal","1" => "unformatted","2"  => "abnormal","3"  => "smartfailed","4"  => "mismatch","5"=> "idle","6"  => "notonline","10"=> "reparing","11"=> "formatting"   );
my %disk_status_status = ( "0"  => "OK","1"  => "UNKNOWN","2" => "CRITICAL","3"  => "WARNING","4"  => "WARNING","5"  => "OK","6"  => "CRITICAL","10"  => "WARNING","11"  => "OK");
sub print_help() {
	printf "check SNMP HiKVision.\n";
	printf "\tcheck system, \n";
	printf "\tcheck disk status\n";
	printf "\tcheck volume raid status\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -i                  ifindex\n";
	print " -w                  warning temperature in Celsius (default: 50)\n";
	print " -c                  critical temperature in Celsius (default: 70)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
    "T=s" => \$type,"type=s" => \$type,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#sysDescr INFORMATION
my $resultat = $snmp->get_request("$oid_sysDescr.0");
if (!defined($resultat)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community,
		);
		$resultat = $snmp->get_request("$oid_sysDescr");
		if (!defined($resultat)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
   		$snmp->close;
  		 exit $ERRORS{"UNKNOWN"};
	}
}
my $description= $$resultat{"$oid_sysDescr.0"};

if($type  eq "status")
{
    my $dev_type = getsnmpOID($snmp,"$oid_type_device");
    print "Device type is ";
    if($dev_type eq '1')
    {
        my $serial = getsnmpOID($snmp,"$oid_serial");
        print "DVR (SN: $serial)";
    }
    elsif($dev_type eq '2')
    {
        print "NVR";
    }
    elsif($dev_type eq '3')
    {
        print "IPC";
    }
    else
    {
        print "unknown";
    }
    print ".\n";
    my $state = "OK";
    my $status = getsnmpOID($snmp,"$oid_device_state");
    my $message="unknown (255)";
    if($status eq '1')
    {
        $state ="OK";
        $message="online";
    }
    elsif($status eq '2')
    {
        $state ="CRITICAL";
        $message="absent";
    }
    else
    {
        $state ="UNKNOWN";
    }
    print "$state : device state is $message\n";
    $return = priority_state($return,$ERRORS{$state});
    $state = "OK";
    $status = getsnmpOID($snmp,"$oid_is_online");
    $message="unknown (255)";
    if($status eq '1')
    {
        $state ="OK";
        $message="online";
    }
    elsif($status eq '2')
    {
        $state ="CRITICAL";
        $message="absent";
    }
    else
    {
        $state ="UNKNOWN";
    }
    print "$state : HikOnline is $message\n";
    
    $return = priority_state($return,$ERRORS{$state});
}
elsif($type  eq "mem")
{
    my $memory = getsnmpOID($snmp,"$oid_mem_usage");
    my $state = "OK";
    my $message="< $warning%";
    if($memory > $critical)
    {
        $state ="CRITICAL";
        $message="> $critical%";
    }
    elsif($memory > $warning)
    {
        $state ="WARNING";
        $message="> $warning%";
    }
    print "$state : memory is used $memory% $message|mem=$memory\n";
    $return = priority_state($return,$ERRORS{$state});
}
elsif($type  eq "disk")
{
    my $result = $snmp->get_table(Baseoid => $oid_disk_volume);
    foreach my $key ( keys %$result) 
    {	
        #on récupère l'index
        if($key=~ /([0-9]*)\.0$/ )
        {
            my $index =$1;
            my $descr =$$result{"$key"};
            my $status = getsnmpOID($snmp,"$oid_disk_status.$index.0");
            my $state = $disk_status_status{$status};
            my $size = getsnmpOID($snmp,"$oid_disk_size.$index.0");
            print "$state : $descr is ".$disk_status_text{$status}.". Disk size: $size MB\n";
            $return = priority_state($return,$ERRORS{$state});
        }
    }

}
exit($return);

