#!/usr/bin/perl -w 
############################## check_snmp_hikvision_camera.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_hikvision_camera.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning=80;
my $critical=90;
my $type = 'temp';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";


#SENSORS
my $oid_snmp_cpu_usage = ".1.3.6.1.4.1.39165.1.7.0";
my $oid_snmp_disk_usage = ".1.3.6.1.4.1.39165.1.9.0";
my $oid_snmp_disk_size = ".1.3.6.1.4.1.39165.1.8.0";
my $oid_snmp_mem_usage = ".1.3.6.1.4.1.39165.1.11.0";
my $oid_snmp_mem_size = ".1.3.6.1.4.1.39165.1.10.0";
# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password

sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub print_help() {
	printf "check snmp HikVision Caméra.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -T                  Test type (cpu,mem,disk)\n";
	print " -w                  warning threshold\n";
	print " -c                  critical threshold\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols,   
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd)) 
{
    $snmp_version =3;
}
if ($snmp_version == 3) 
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass)) 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -timeout          => 2
        );  
    }
    else 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysObjectOID");


#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
{
    $Sysoid_value = $1;
}
if($type eq 'cpu')
{
    my $value = getsnmpOID($snmp,"$oid_snmp_cpu_usage"); 
    if( $value=~ /([0-9]*)\s+/)
    {
        my $message = "< $warning%";
        my $state = "OK";
        $value = $1;
        if($value > $critical)
        {
            $state = "CRITICAL";
            $message = "> $critical%";
        }
        elsif($value > $warning)
        {
            $state = "WARNING";
            $message = "> $warning%";
        }
        print "$state: CPU used $value% $message|cpu=$value";
        $return = priority_state($return,$ERRORS{$state});
    }
    else
    {
        print "UNKNOWN: bad format CPU data";
        $return = priority_state($return,$ERRORS{'UNKNOWN'});        
    }
}
elsif($type eq 'mem')
{
    my $value = getsnmpOID($snmp,"$oid_snmp_mem_usage"); 
    if( $value=~ /([0-9]*)\s+/)
    {
        my $message = "< $warning%";
        my $state = "OK";
        $value = $1;
        if($value > $critical)
        {
            $state = "CRITICAL";
            $message = "> $critical%";
        }
        elsif($value > $warning)
        {
            $state = "WARNING";
            $message= "> $warning%";
        }
        print "$state: RAM used $value% $message|ram=$value";
        $return = priority_state($return,$ERRORS{$state});
    }
    else
    {
        print "UNKNOWN: bad format CPU data";
        $return = priority_state($return,$ERRORS{'UNKNOWN'});        
    }
}
elsif($type eq 'disk')
{
    my $value = getsnmpOID($snmp,"$oid_snmp_disk_usage"); 
    if( $value=~ /([0-9]*)\s+/)
    {
        my $message = "< $warning%";
        my $state = "OK";
        $value = $1;
        if($value > $critical)
        {
            $state = "CRITICAL";
            $message = "> $critical%";
        }
        elsif($value > $warning)
        {
            $state = "WARNING";
            $message= "> $warning%";
        }
        print "$state: disk used $value% $message|disk=$value";
        $return = priority_state($return,$ERRORS{$state});
    }
    else
    {
        print "UNKNOWN: bad format CPU data";
        $return = priority_state($return,$ERRORS{'UNKNOWN'});        
    }
}
exit($return);
