#!/usr/bin/perl -w
############################## check_snmp_poe.pl ########
my $Version='1.0';
# Date : Jan 15 2021
# Author  : Julien George
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

#use Data::Dumper;

my $PROGNAME='check_snmp_poe.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning='';
my $critical='';
my $type = 'env';

# Mib Standard
my $oid_sysDescr = "1.3.6.1.2.1.1.1.0";
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";

my $standard_MPS 		= "1.3.6.1.2.1.105.1.3.1.1.2.1"; # Maximum Power Supply    Watts
my $standard_PowerConsu = ".1.3.6.1.2.1.105.1.3.1.1.4.1"; # Total Power Consumption Watts
my $standard_LevelAlarm = ".1.3.6.1.2.1.105.1.3.1.1.5.1"; # Power Over-Usage in % limite en % qd il envoit une alarme

# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password

sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub print_help() {
	printf "check snmp HP switch.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning threshold\n";
	print " -c                  critical threshold\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) {
    print "ERROR : You forget to specify -H, --host option\n\n";
    print_help();
    exit $ERRORS{'UNKNOWN'};
}

my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd))
{
    $snmp_version =3;
}
if ($snmp_version == 3)
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass))
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -timeout          => 2
        );
    }
    else
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}

#on interroge en SNMP pour récupérer les informations
my $result = $snmp->get_request("$oid_sysObjectOID");

#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result))
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}

my $total = getsnmpOID($snmp,$standard_MPS);
my $used = getsnmpOID($snmp,$standard_PowerConsu);

if ($total =~ "noSuch" || $used =~ "noSuch"){
    print "ERROR : no reponse from agent SNMP for POE standard OIDs\n";
	exit $ERRORS{'UNKNOWN'};
}

my $percent = $used/$total*100;

my $level_alarm	= getsnmpOID($snmp,$standard_LevelAlarm);
if ($level_alarm > 100 ){ ## La plupart du temps, la valeur est en %. Mais elle est parfois en Watts (Exemple D-LINK DGS 1210)
	$level_alarm = $level_alarm/$total*100;
}

if( !$warning ){
	$warning = $level_alarm;
}
if( !$critical ){
	$critical = (100-$level_alarm)/2 + $level_alarm;
}

if( $warning > $critical ){
	print "ERROR : -w, --warning '$warning' treshold must be lower than -c, --critical '$critical' treshold\n\n";
	print_help();
	exit $ERRORS{'UNKNOWN'};
}

my $message="(< $warning)";
my $state = 'OK';
if($percent >= $critical)
{
	$state='CRITICAL';
	$return = priority_state($return,$ERRORS{CRITICAL});
	$message="(> $critical)";
}
elsif($percent >= $warning)
{
	$state='WARNING';
	$return = priority_state($return,$ERRORS{WARNING});
	$message="(> $warning)";
}

printf("%s : POE usage %d/%d Watts %.1f%% %s (System level alarm %.1f%%);w:$warning;c:$critical\n",$state,$used,$total,$percent,$message,$level_alarm);
exit($return);
