#!/usr/bin/perl
use Net::SNMP;
use Getopt::Long;
use Data::Dumper;
use Switch;
use strict;

my $PROGNAME='check_snmp_stormshield_current_vpns.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my @PRIORITIES_STATES=('CRITICAL','WARNING','UNKNOWN','OK','IGNORE');
my $return=$ERRORS{OK};

my $o_timeout=  5; 

my $oid_sysDescr = "1.3.6.1.2.1.1.1.0";

my $oid_IPSECStatsSPDIn='.1.3.6.1.4.1.11256.1.13.1.1.0';
my $oid_IPSECStatsSPDOut='.1.3.6.1.4.1.11256.1.13.1.2.0';
my $oid_IPSECStatsSADLarval='.1.3.6.1.4.1.11256.1.13.2.1.0';
my $oid_IPSECStatsSADMature='.1.3.6.1.4.1.11256.1.13.2.2.0';
my $oid_IPSECStatsSADDying='.1.3.6.1.4.1.11256.1.13.2.3.0';
my $oid_IPSECStatsSADDead='.1.3.6.1.4.1.11256.1.13.2.4.0';

my $snmp_version = 2;

sub print_usage {
    print "Usage: $PROGNAME [-v] -H <host> -C <snmp_community> -w INT_MIN,[INT_MAX] -c INT_MIN,[INT_MAX] [-t <timeout>]\n";
}

sub help {
   print "\nSNMP Stormshield current established VPNs connections\n";
   print "Author: GEORGE Julien\n\n";
   print_usage();
   print <<EOT;
-h, --help
   print this help message
-v, --verbose
   print extra debugging information (and lists all storages)

-H, --hostname=HOST
   name or IP address of host to check
-C, --community=COMMUNITY NAME
   community name for the host's SNMP agent (implies SNMP v1 or v2c with option)
-t, --timeout=INTEGER
   timeout for SNMP in seconds (Default: 5)
-w, --warning=INT_MIN,INT_MAX
	min et max for warning
-c, --critical=INT_MIN,INT_MAX 
	min et max for critical
EOT
}
my $o_verb;

sub verb { my $t=shift; print $t,"\n" if defined($o_verb) ; }

my $o_help;
my $o_warning;
my $o_critical;

my $o_host;
my $o_community;

my $min_warn;
my $min_crit;
my $max_warn;
my $max_crit;

sub check_options {
    Getopt::Long::Configure ("bundling");
    GetOptions(
		'v'		=> \$o_verb,		'verbose'		=> \$o_verb,
        'h'     => \$o_help,    	'help'        	=> \$o_help,
        'H:s'   => \$o_host,		'hostname:s'	=> \$o_host,
		'C:s'   => \$o_community,	'community:s'	=> \$o_community,
		't:i'   => \$o_timeout,     'timeout:i'     => \$o_timeout,
		'w:s'	=> \$o_warning,		'warning:s'		=> \$o_warning,
		'c:s'	=> \$o_critical,	'critical:s'	=> \$o_critical,
    );

    if (defined($o_help) ) { help(); exit $ERRORS{"UNKNOWN"}};
  
    # Check compulsory attributes
    if ( !defined($o_host) ) { 
		print "You forget to specify host (-h)\n";
		print_usage(); exit $ERRORS{"UNKNOWN"}
	};
	if ( !defined($o_warning) || !defined($o_critical) ) {
		print "You forget to specifiy warning (-w) and/or critical values(-c)\n";
		print_usage(); exit $ERRORS{"UNKNOWN"};
	}
	
	($min_warn, $max_warn) = split(/,/,$o_warning);
	($min_crit, $max_crit) = split(/,/,$o_critical);

	if( $min_warn < $min_crit ){
		print("Minimum value for warning : $min_warn must be greater than minimum value for critical : $min_crit\n");
		print_usage(); exit $ERRORS{"UNKNOWN"};
	}

	if( defined($max_warn) && $max_warn < $min_warn ){
		print("Maximum value for warning : $min_warn must be greater than minimum value for warning : $max_warn\n");
		print_usage(); exit $ERRORS{"UNKNOWN"};
	}

	if( defined($max_crit) && $max_crit < $min_warn ){
		print("Maximum value for critical : $max_crit must be greater than minimum value for warning : $min_warn\n");
		print_usage(); exit $ERRORS{"UNKNOWN"};
	}

	if( defined($max_warn) && defined($max_crit) && $max_crit < $max_warn ) {
		print("Maximum value for critical : $max_crit must be greater than maximum value for warning : $max_crit\n");
		print_usage(); exit $ERRORS{"UNKNOWN"};
	} 
}

check_options();

# Check gobal timeout
if (defined($TIMEOUT)) {
  verb("Alarm at $TIMEOUT");
  alarm($TIMEOUT);
} else {
  verb("no timeout defined : $o_timeout + 10");
  alarm ($o_timeout+10);
}

my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $o_host,
		-version => 'snmpv2c',
		-community => $o_community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $o_host,
		-version => 'snmpv1',
		-community => $o_community,
		-timeout=>2
	);
}

#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $o_host,
			-version => 'snmpv1',
			-community => $o_community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}

sub snmpget{
	my ( $oid ) = @_ ;
	my $snmpresult = $snmp->get_request("$oid");
	return $$snmpresult{"$oid"};
}

#my $in = snmpget($oid_IPSECStatsSPDIn);
#my $out = snmpget($oid_IPSECStatsSPDOut);

my $larval=snmpget($oid_IPSECStatsSADLarval);
my $mature=snmpget($oid_IPSECStatsSADMature);
my $dying=snmpget($oid_IPSECStatsSADDying);
my $dead=snmpget($oid_IPSECStatsSADDead);

my $message = "$mature matures/running VPNs + ($larval larval, $dying dying, $dead dead)|$min_crit,$min_warn,$max_warn,$max_crit";

my $status="OK";
if( defined($max_crit) && $mature >= $max_crit ){
	$status = "CRITICAL";
	print "$status - $message\n";
	exit($ERRORS{$status});
} 
elsif( $mature <= $min_crit ){
	$status = "CRITICAL";
	print "$status - $message\n";
	exit($ERRORS{$status});
}

if( defined($max_warn) && $mature >= $max_warn ){
	$status = "WARNING";
	print "$status - $message\n";
	exit($ERRORS{$status});
} 
elsif( $mature <= $min_warn ){
	$status = "WARNING";
	print "$status - $message\n";
	exit($ERRORS{$status});
}

#print "$in/$out $larval $mature $dying $dead\n";

print "$status - $message\n";
exit($ERRORS{$status});
