#!/usr/bin/perl
use Net::SNMP;
use Getopt::Long;
use Data::Dumper;
use Switch;
use strict;

my $PROGNAME='check_snmp_stormshield_ha.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my @PRIORITIES_STATES=('CRITICAL','WARNING','UNKNOWN','OK');
my $return=$ERRORS{OK};

my $o_timeout=  5; 

my $oid_sysDescr = "1.3.6.1.2.1.1.1.0";

my $oid_NbNode='.1.3.6.1.4.1.11256.1.11.1.0';
my $oid_NbDeadNode='.1.3.6.1.4.1.11256.1.11.2.0';
my $oid_NbActiveNode='.1.3.6.1.4.1.11256.1.11.3.0';
my $oid_NbHALinks='.1.3.6.1.4.1.11256.1.11.5.0';
my $oid_NbFaultyHALinks='.1.3.6.1.4.1.11256.1.11.6.0';

my $mib_ha='1.3.6.1.4.1.11256.1.11.7';
my $idx_index=1;
my $idx_FwSerial=2;
my $idx_Online=3;
my $idx_Model=4;
my $idx_Version=5;
my $idx_HALicence=6;
my $idx_HAQuality=7;
my $idx_HAPriority=8;
my $idx_HAStatusForced=9;
my $idx_HAActive=10;
my $idx_Uptime=11;


my $snmp_version = 2;

sub print_usage {
	print "Usage: $PROGNAME [-v] -H <host> -C <snmp_community> [-t <timeout>]\n";
    	print "Usage: $PROGNAME [-v] -H <host> -C <snmp_community> [-a MIN_WARNING[,MIN_CRITICAL]] [-t <timeout>]\n";
	print "Usage: $PROGNAME [-v] -H <host> -C <snmp_community> [-d MAX_WARNING[,MAX_CRITICAL]] [-t <timeout>]\n";
	print "Usage: $PROGNAME [-v] -H <host> -C <snmp_community> [-l MAX_WARNING[,MAX_CRITICAL]] [-t <timeout>]\n";
}

sub help {
   print "\nSNMP Stormshield HA\n";
   print "Author: GEORGE Julien\n\n";
   print_usage();
   print <<EOT;
-h, --help
   print this help message
-v, --verbose
   print extra debugging information (and lists all storages)

-H, --hostname=HOST
   name or IP address of host to check
-C, --community=COMMUNITY NAME
   community name for the host's SNMP agent (implies SNMP v1 or v2c with option)
-t, --timeout=INTEGER
   timeout for SNMP in seconds (Default: 5)
-a, --active=MIN_WARNING,MIN_CRITICAL
	check minimal active nodes for warning and critical
-d, --dead=MAX_WARNING,MAX_CRITICAL
	check maximal dead nodes for warning and critical
-l, --link=MAX_WARNING,MAX_CRITICAL
	check maximum faulty nodes for warning and critical

Examples :
	$PROGNAME -H 192.168.1.3 -C public -a 0,0 => return critical if 0 active nodes.
	$PROGNAME -H 192.168.1.3 -C public -d 1,2 => return warning if 1 node are dead or critical for 2 nodes
	$PROGNAME -H 192.168.1.3 -C public -l 1 => return warning if 1 link are faulty
	$PROGNAME -H 192.168.1.3 -C -a 0,0 -d 1,1 -l 1,2 => check multiple
EOT
}
my $o_verb;

sub verb { my $t=shift; print $t,"\n" if defined($o_verb) ; }

my $o_host;
my $o_community;
my $o_system;
my $o_help;
my $o_host;

my $o_active;
my $o_dead;
my $o_link;

sub calcul_state{
	my ( $a, $b ) = @_;
	$a = uc $a;
	$b = uc $b;	 

	foreach my $prio (@PRIORITIES_STATES){
		if( $prio eq $a || $prio eq $b ) {
			return $prio;
		}
	}
}

sub check_options {
    Getopt::Long::Configure ("bundling");
    GetOptions(
		'v'		=> \$o_verb,		'verbose'		=> \$o_verb,
        'h'     => \$o_help,    	'help'        	=> \$o_help,
        'H:s'   => \$o_host,		'hostname:s'	=> \$o_host,
		'C:s'   => \$o_community,	'community:s'	=> \$o_community,
		'a:s'	=> \$o_active,		'active:s'		=> \$o_active,
		'd:s'	=> \$o_dead,		'dead:s'		=> \$o_dead,
		'l:s'	=> \$o_link,		'link:s'		=> \$o_link,
		't:i'   => \$o_timeout,     'timeout:i'     => \$o_timeout,
    );

    if (defined($o_help) ) { help(); exit $ERRORS{"UNKNOWN"}};
  
    # Check compulsory attributes
    if ( !defined($o_host) ) { print_usage(); exit $ERRORS{"UNKNOWN"}};
}

check_options();

# Check gobal timeout
if (defined($TIMEOUT)) {
  verb("Alarm at $TIMEOUT");
  alarm($TIMEOUT);
} else {
  verb("no timeout defined : $o_timeout + 10");
  alarm ($o_timeout+10);
}

my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $o_host,
		-version => 'snmpv2c',
		-community => $o_community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $o_host,
		-version => 'snmpv1',
		-community => $o_community,
		-timeout=>2
	);
}

#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $o_host,
			-version => 'snmpv1',
			-community => $o_community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}

sub snmpget{
	my ( $oid ) = @_ ;
	my $snmpresult = $snmp->get_request("$oid");
	return $$snmpresult{"$oid"};
}

my $nodes = snmpget($oid_NbNode);
my $deadNodes = snmpget($oid_NbDeadNode);
my $activeNodes=snmpget($oid_NbActiveNode);
my $haLinks =snmpget($oid_NbHALinks);
my $faultyLinks=snmpget($oid_NbFaultyHALinks);

print "STATS => Active nodes : $activeNodes/$nodes | Dead nodes : $deadNodes/$nodes | Faulty ha links : $faultyLinks/$haLinks\n";

my $resultat = $snmp->get_table(Baseoid => $mib_ha);

my $idx=1;

while( defined( $resultat->{"$mib_ha.$idx.$idx_index.0"} ) ) {
	my $field=0;
	while( defined( $resultat->{"$mib_ha.$idx.$idx_index.$field"} ) ) {
		my $sn = $resultat->{"$mib_ha.$idx.$idx_FwSerial.$field"};
		my $online = $resultat->{"$mib_ha.$idx.$idx_Online.$field"};
		my $model = $resultat->{"$mib_ha.$idx.$idx_Model.$field"};
		my $version = $resultat->{"$mib_ha.$idx.$idx_Version.$field"};
		my $halicence = $resultat->{"$mib_ha.$idx.$idx_HALicence.$field"};
		my $haquality = $resultat->{"$mib_ha.$idx.$idx_HAQuality.$field"};
		my $hapriority = $resultat->{"$mib_ha.$idx.$idx_HAPriority.$field"};
		my $hastatus = $resultat->{"$mib_ha.$idx.$idx_HAStatusForced.$field"};
		my $haactive = $resultat->{"$mib_ha.$idx.$idx_HAActive.$field"};
		my $uptime = $resultat->{"$mib_ha.$idx.$idx_Uptime.$field"};

		my $days = int( $uptime / 86400 );
		$uptime-=$days*86400;
		my $hours= int( ( $uptime ) / 3600 );
		$uptime-=$hours*3600;
		my $min  = int( ( $uptime / 60 ) );
		$uptime-=$min*60;
		my $sec = $uptime;
		 
	
		printf("Node %d %-17s ( V%-6s) : %-6s up since %04dd %02d:%02d:%02d\n", $field,$sn,$version,$halicence,$days,$hours,$min,$sec);
		
		$field++;
	}		
	$idx++;
}

if ( !defined($o_active) && !defined($o_dead) && !defined($o_link)){
	print "This check can't return WARNING or CRITICAL without options -a, -d or -l\n";
	exit($ERRORS{"OK"});
}

my $status='OK';
my $ret='OK';
if( defined($o_active) ){
	my ($warn, $critic) = split(/,/,$o_active);

	if( defined $critic && $activeNodes <= $critic ){
		$status = 'CRITICAL';
		printf("%-8s - $activeNodes actives nodes <= $critic\n",$status);
	}
	elsif( $activeNodes <= $warn ){
		$status = 'WARNING';
		printf("%-8s - $activeNodes actives nodes <= $warn\n",$status);
	}
	else{
		printf("%-8s - $activeNodes actives nodes > $warn\n",$status);
	}
}

$ret = calcul_state($status,$ret);

if( defined($o_dead) ){
	my ($warn, $critic) = split(/,/,$o_dead);

	if( defined $critic && $deadNodes >= $critic ){
		$status = 'CRITICAL';
		printf("%-8s - $deadNodes dead nodes >= $critic\n",$status);
	}
	elsif( $deadNodes >= $warn ){
		$status = 'WARNING';
		printf("%-8s - $deadNodes dead nodes >= $warn\n",$status);
	}
	else{
		printf("%-8s - $deadNodes dead nodes < $warn\n",$status);
	}
}
$ret = calcul_state($status,$ret);

if( defined($o_link) ){
	my ($warn, $critic) = split(/,/,$o_link);

	if( defined $critic && $faultyLinks >= $critic ){
		$status = 'CRITICAL';
		printf("%-8s - $faultyLinks faulty links >= $critic\n",$status);
	}
	elsif( $faultyLinks >= $warn ){
		$status = 'WARNING';
		printf("%-8s - $faultyLinks faulty links >= $warn\n",$status);
	}
	else{
		printf("%-8s - $faultyLinks faulty links < $warn\n",$status);
	}
}
$ret = calcul_state($status,$ret);



exit($ERRORS{$status});
