#!/usr/bin/perl
use Net::SNMP;
use Getopt::Long;
use Data::Dumper;
use Switch;
use strict;

my $PROGNAME='check_snmp_stormshield_modules.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'IGNORE'=>3,'UNKNOWN'=>3,'DEPENDENT'=>4);
my @PRIORITIES_STATES=('CRITICAL','WARNING','UNKNOWN','OK','IGNORE');
my $return=$ERRORS{OK};

my $mib_autoupdate='.1.3.6.1.4.1.11256.1.9.1';
my $idx_index='1';
my $idx_system='2';
my $idx_state='3';
my $idx_date='4';

my $o_timeout=  5;  
my $snmp_version = 2;


my $D_OK='uptodate';
my $D_WARNING='outofdate';
my $D_CRITICAL='disabled';
my $D_UNKNOWN='';
my $D_IGNORE='notavailable|neverstarted';

my @SERVICES_MIB_STATUS=('uptodate','disabled','notavailable','outofdate','neverstarted');

my %MibToCheckStatus;
$MibToCheckStatus{'notavailable'} = 'ignore';
$MibToCheckStatus{'disabled'} = 'ignore';
$MibToCheckStatus{'outofdate'} = 'ignore';
$MibToCheckStatus{'uptodate'} = 'ignore';
$MibToCheckStatus{'neverstarted'} = 'ignore';

sub calcul_state{
	my ( $a, $b ) = @_;
	$a = uc $a;
	$b = uc $b;	 

	foreach my $prio (@PRIORITIES_STATES){
		if( $prio eq $a || $prio eq $b ) {
			return $prio;
		}
	}
}

sub isValidMibStatus{
	my ( $p_status ) = @_;
	$p_status = lc $p_status;
	
	foreach my $mib_srv (@SERVICES_MIB_STATUS){
		if( $p_status == $mib_srv ) {
			return 1;
		}
	}
	return 0;
}

sub groupMibStatus{
	my ( $statusMib_list, $checked_status ) = @_;

	foreach my $statusMib (split /,/, $statusMib_list){
		if(isValidMibStatus($statusMib)){
			$MibToCheckStatus{$statusMib} = $checked_status;
		}
	}
}

sub print_usage {
    print "Usage: $PROGNAME [-v] -H <host> -C <snmp_community> [-2] [-t <timeout>]\n";
}

sub help {
   print "\nSNMP Stormshield netasq updates\n";
   print "Author: GEORGE Julien\n\n";
   print_usage();
   print <<EOT;
-h, --help
   print this help message
-v, --verbose
   print extra debugging information (and lists all storages)

-H, --hostname=HOST
   name or IP address of host to check
-C, --community=COMMUNITY NAME
   community name for the host's SNMP agent (implies SNMP v1 or v2c with option)
-t, --timeout=INTEGER
   timeout for SNMP in seconds (Default: 5)
-s, --system=REGEX
   insensitive regex that matches names of system to check  (Match all by default)

-i, --ignore=LIST
	comma separtad list of status that are ignored. (Default : notavailable,neverstarted)
-o, --ok=LIST
	comma separated list of status that cause warning (Default : uptodate)
-w, --warning=LIST
	comma separated list of status that cause warning (Default : outofdate)
-c, --critical=LIST 
	comma separated list of status that cause critical (Default : disabled|)
-u, --unknown=LIST
	comma separated list of status that cause unknown (Default : none)

Possible states are : uptodate, disabled, notavailable, neverstarted, outofdate.
Key none is for ignore the monitoring state.
EOT
}
my $o_verb;

sub verb { my $t=shift; print $t,"\n" if defined($o_verb) ; }

my $o_ok;
my $o_ignore;
my $o_warning;
my $o_critical;
my $o_unknown;

my $o_host;
my $o_community;
my $o_system;
my $o_version2;
my $o_help;
my $o_host;

sub check_options {
    Getopt::Long::Configure ("bundling");
    GetOptions(
		'v'		=> \$o_verb,		'verbose'		=> \$o_verb,
        'h'     => \$o_help,    	'help'        	=> \$o_help,
        'H:s'   => \$o_host,		'hostname:s'	=> \$o_host,
		'C:s'   => \$o_community,	'community:s'	=> \$o_community,
		't:i'   => \$o_timeout,     'timeout:i'     => \$o_timeout,
		's:s'	=> \$o_system,		'system:s'		=> \$o_system,
		'o:s'	=> \$o_ok,			'ok:s'			=> \$o_ok,
		'i:s'	=> \$o_ignore,		'ignore:s'		=> \$o_ignore,
		'w:s'	=> \$o_warning,		'warning:s'		=> \$o_warning,
		'c:s'	=> \$o_critical,	'critical:s'	=> \$o_critical,
		'u:s'	=> \$o_unknown,		'unknown:s'		=> \$o_unknown,
		'2'     => \$o_version2,    'v2c'           => \$o_version2,
    );

    if (defined($o_help) ) { help(); exit $ERRORS{"UNKNOWN"}};
  
    # Check compulsory attributes
    if ( !defined($o_host) ) { print_usage(); exit $ERRORS{"UNKNOWN"}};

	groupMibStatus($D_IGNORE,'ignore');
	groupMibStatus($D_OK,'ok');
	groupMibStatus($D_UNKNOWN,'unknown');
	groupMibStatus($D_WARNING,'warning');
	groupMibStatus($D_CRITICAL,'critical');

	groupMibStatus($o_ignore,'ignore');
	groupMibStatus($o_ok,'ok');
	groupMibStatus($o_unknown,'unknown');
	groupMibStatus($o_warning,'warning');
	groupMibStatus($o_critical,'critical');	
}

sub translate_received_status{
	my ($received) = @_;
	
	$received = lc $received;

	switch( $received ) {
		case 'uptodate'				{	return $received; 		}
		case 'disabled'				{	return $received; 		}
		case 'not available'		{	return 'notavailable';  }
		case 'never started'		{ 	return 'neverstarted'; 	}
		else 						{	return 'outofdate';		}
	}
}

check_options();

# Check gobal timeout
if (defined($TIMEOUT)) {
  verb("Alarm at $TIMEOUT");
  alarm($TIMEOUT);
} else {
  verb("no timeout defined : $o_timeout + 10");
  alarm ($o_timeout+10);
}

# Connect to host
my ($session,$error);
if ($snmp_version == 2) {
	# SNMPv2 Login
	verb("SNMP v2c login");
	  ($session, $error) = Net::SNMP->session(
	 -hostname  => $o_host,
	 -version   => 2,
	 -community => $o_community,
	 -timeout   => $o_timeout
	);
} else {
  # SNMPV1 login
  verb("SNMP v1 login");
  ($session, $error) = Net::SNMP->session(
	-hostname  => $o_host,
	-version   => 1,
	-community => $o_community,
	-timeout   => $o_timeout
  );
}

if (!defined($session)) {
   printf("ERROR: %s.\n", $error);
   exit $ERRORS{"UNKNOWN"};
}

my $resultat = $session->get_table(Baseoid => $mib_autoupdate);

if (!defined($resultat))
{
		verb(sprintf("ERROR: %s.\n", $session->error));
		$session->close;

        if(defined ($o_version2))
        {
				verb("Try SNMP v1");
                ($session, $error) = Net::SNMP->session(
                        -hostname => $o_host,
						-version   => 1,
                        -community => $o_community,
						-timeout   => $o_timeout
                );

				if (!defined($session)) {
					   printf("ERROR: %s.\n", $error);
					   exit $ERRORS{"UNKNOWN"};
				}

                $resultat = $session->get_table(Baseoid => $mib_autoupdate);
                if (!defined($resultat))
                {
                        printf("UNKNOWN ERROR: Description table : %s.\n", $session->error);
                        $session->close;
                        exit $ERRORS{"UNKNOWN"};
                }
        }
        else
        {
                printf("UNKNOWN ERROR: Description table : %s.\n", $session->error);
                $session->close;
                exit $ERRORS{"UNKNOWN"};
        }
}
else{
	$session->close;
}

#print Dumper($resultat); 

my $idx=1;
my $ret='IGNORE';
while( defined( $resultat->{"$mib_autoupdate.$idx.$idx_index.0"} ) ) {
	my $field=0;
	while( defined( $resultat->{"$mib_autoupdate.$idx.$idx_index.$field"} ) ) {
		my $name = $resultat->{"$mib_autoupdate.$idx.$idx_system.$field"};
		my $state = $resultat->{"$mib_autoupdate.$idx.$idx_state.$field"};
		my $date = $resultat->{"$mib_autoupdate.$idx.$idx_date.$field"};

		if ( !defined ( $o_system ) || $name =~ /$o_system/i || $o_system=~'/all/i' ){
			if( length "$date" > 0 ) { $date = "since $date"; }
			
			$state = translate_received_status($state);
			my $esia_state = uc $MibToCheckStatus{$state};
			$ret = calcul_state($ret,$esia_state);

			printf("%-8s - %d. %-20s : %-15s %s\n", $esia_state,$idx,$name, $state, $date);
		}
		$field++;
	}		
	$idx++;
	exit(%ERRORS{$ret});
}
