#!/usr/bin/perl
use strict;
use Getopt::Long;
use Data::Dumper;

# Specific
our $TIMEOUT;
our %ERRORS;
eval 'use utils qw(%ERRORS $TIMEOUT)';
if ($@) {
 $TIMEOUT = 10;
 %ERRORS = ('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
}

our $no_snmp=0;
eval 'use Net::SNMP';
if ($@) {
  $no_snmp=1;
}

# SNMP OID
my $oid_svSvcEntry = '.1.3.6.1.4.1.77.1.2.3.1';
my $oid_svSvcName  = '.1.3.6.1.4.1.77.1.2.3.1.1';
my $oid_svSvcInstalledState  = '.1.3.6.1.4.1.77.1.2.3.1.2';
my $oid_svSvcOperatingState  = '.1.3.6.1.4.1.77.1.2.3.1.3';
my $oid_svSvcUninstalled  = '.1.3.6.1.4.1.77.1.2.3.1.4';


my %label_svSvcInstalledState = ( 1 => 'uninstalled', 2=> 'install-pending', 3=> 'uninstall-pending', 4=> 'installed');
my %label_svSvcOperatingState = ( 1 => 'active', 2=> 'continue-pending', 3=> 'pause-pending', 4=> 'paused');
my %label_svSvcUninstalled = ( 1 => 'cannot-be-uninstalled', 2=> 'can-be-uninstalled');

my $o_host = 		undef; 	# hostname
my $o_timeout=  	undef;  # Timeout (Default 10)
my $o_help=		undef; 	# wan't some help ?
my $o_warn=             undef;  # WARNING alert if system has been up for < specified number of minutes
my $o_crit=             undef;  # CRITICAL alert if system has been up for < specified number of minutes
my $o_community =	undef; 	# community

my $o_descr = undef;
my $o_noreg = undef;
my $o_number = undef;
my $o_showall = undef;
my $o_brief = undef;

my @o_descrL = 	undef;

sub print_usage {
    print "Usage: $0 -H <host> -C <snmp_community> -s\n";
    print "Usage: $0 -H <host> -C <snmp_community> -n [-N <number>] [-r] [-s|-b]\n";
}

sub isnnum { # Return true if arg is not a number
  my $num = shift;
  if ( $num =~ /^(\d+\.?\d*)|(^\.\d+)$/ ) { return 0 ;}
  return 1;
}

sub isnotnum { # Return true if arg is not a number
  my $num = shift;
  if ( $num =~ /^-?(\d+\.?\d*)|(^\.\d+)$/ ) { return 0 ;}
  return 1;
}

sub help {
   print "\check_snmp_win_service Plugin\n";
   print_usage();
   print <<EOT;

Help Options:
 -h, --help

SNMP Protocol Options:
 -H, --hostname=HOST
   name or IP address of host to check (if not localhost)
 -C, --community=COMMUNITY NAME
   community name for the SNMP agent (used with v1 or v2c protocols)
 -t, --timeout=INTEGER
   timeout for SNMP in seconds (Default: 15)

Matching Options :
 -n, --name=NAME[,NAME2...]
   Comma separated names of services (perl regular expressions can be used for every one).
 -r, --noregexp
   Do not use regexp to match NAME in service description.

Result Options :
 -N, --number=<n>
   Compare matching services with <n> instead of the number of names provided.

Output Options :
 -s, --showall
   Show all services in the output, instead of only the non-active ones.
 -b, --brief
   Show only first line.

This script will return  :
 UNKNOWN  if -n, --name missed.
 CRITICAL if at least one of service is non active.
 WARNING  if -N, --number if present and different from all services found.
 OK       else case.
EOT
}

sub check_options {
    Getopt::Long::Configure ("bundling");
	GetOptions(
        'h'     => \$o_help,    	'help'        	=> \$o_help,
        'H:s'   => \$o_host,		'hostname:s'	=> \$o_host,
        'C:s'   => \$o_community,	'community:s'	=> \$o_community,
        't:i'   => \$o_timeout,    	'timeout:i'	    => \$o_timeout,

        'n:s'   => \$o_descr,		'name:s'	    => \$o_descr,
        'r'     => \$o_noreg,       'noregexp'      => \$o_noreg,

        'N:i'   => \$o_number,      'number:i'      => \$o_number,

        'b'     => \$o_brief,  	    'brief'         => \$o_brief,
	    's'     => \$o_showall,  	'showall'       => \$o_showall,
    );
  if (defined ($o_help) ) { help(); exit $ERRORS{"UNKNOWN"}};

    if (!defined($o_community) ) {
	    print "Why are you specifying hostname without SNMP parameters?\n"; print_usage(); exit $ERRORS{"UNKNOWN"};
	 }
    else {
         if (!defined($o_host)) {
             print "Hostname required for SNMP check.\n"; print_usage(); exit $ERRORS{"UNKNOWN"};
         }
    }

    # check snmp information
    if (defined($o_timeout) && (isnnum($o_timeout) || ($o_timeout < 2) || ($o_timeout > 60)))
	{ print "Timeout must be >1 and <60 !\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
    if (!defined($o_timeout)) {$o_timeout=$TIMEOUT+5;}


    if ( ! defined($o_descr) && ! defined($o_showall) ) { print_usage(); exit $ERRORS{"UNKNOWN"}};
    @o_descrL=split(/,/,$o_descr);
    foreach my $List (@o_descrL) {
      if ( ! is_pattern_valid ($List) ) { print "Invalid pattern ! ";print_usage(); exit $ERRORS{"UNKNOWN"} }
    }
    if (defined ($o_number)) {
      if (isnotnum($o_number) || ($o_number<0) ) 
        { print "Invalid number of services!\n";print_usage(); exit $ERRORS{"UNKNOWN"}}
    }
}

sub is_pattern_valid { # Test for things like "<I\s*[^>" or "+5-i"
 my $pat = shift;
 if (!defined($pat)) { $pat=" ";} # Just to get rid of compilation time warnings
 return eval { "" =~ /$pat/; 1 } || 0;
}

sub create_snmp_session {
  my ($session,$error);
    # SNMPv2c Login
      ($session, $error) = Net::SNMP->session(
       -hostname  => $o_host,
       -version   => 2,
       -community => $o_community,
       -retries   => 1,
       -timeout   => 2
      );
  if (!defined($session)) {
     printf("ERROR opening session: %s.\n", $error);
     exit $ERRORS{"UNKNOWN"};
  }

  return $session;
}

sub getlist {
    my ($session, $first_oid) = @_;
    my $result=undef;
    my $ret=undef;
    my $next=undef;
    my $key=undef;
    
    $result = $session->get_next_request(-varbindlist=>[ $first_oid ]);
    if (!defined($result)) {
       printf("ERROR: Can not retrieve $oid_svSvcName table: %s.\n", $session->error);
       $session->close;
       exit $ERRORS{"UNKNOWN"};
    }

    #$key = (keys $result)[0];
    $key = (keys %{$result})[0];

    $next = $key;
    $key =~ s/$first_oid//g;

    while(defined($key)){
        $$ret{$key}=$$result{$next};
        $key=undef;

        $result = $session->get_next_request(-varbindlist=>[ $next ]);
        if(defined ($result)){
           # $key = (keys $result)[0];
             $key = (keys %{$result})[0];
            $next = $key;
            $key =~ s/$first_oid//g;
            if( $next eq $key ){
                $key=undef;
            }
        }
    }
    return $ret;
}

sub decode { 
  my $utfstr=shift;
  if (substr($utfstr,0,2) ne "0x") { return $utfstr; }
  my @stringL=split(//,$utfstr);
  my $newstring="";
  for (my $i=2;$i<$#stringL;$i+=2) {
    if ( ($stringL[$i] . $stringL[$i+1]) eq "c3") {
	    $i+=2;$newstring .= ".";
	} else {
	  $newstring .= chr(hex($stringL[$i] . $stringL[$i+1]));
	}
  }
  return $newstring;
}

$SIG{'ALRM'} = sub {
 print "Alarm timeout\n";
 exit $ERRORS{"UNKNOWN"};
};

########## MAIN #######
my $status=0;

check_options();

if (defined($o_timeout)) {
  alarm($o_timeout+5);
}


my $listNames=undef;
my $listInstallStates=undef;
my $listOperatingStates=undef;
my $listUninstalled=undef;

my $session=create_snmp_session();
$listNames = getlist($session,$oid_svSvcName);
$listInstallStates = getlist($session,$oid_svSvcInstalledState);
$listOperatingStates = getlist($session,$oid_svSvcOperatingState);
#$listUninstalled = getlist($session,$oid_svSvcUninstalled);
$session->close;

foreach my $key ( keys %$listNames) {
    $$listNames{$key}=decode($$listNames{$key});
}

my $all_str = '';
##For full stdout
if( defined($o_showall)){
    foreach my $key ( keys %$listNames) {
        $all_str .= $label_svSvcInstalledState{$$listInstallStates{$key}} . " and "; 
        $all_str .= $label_svSvcOperatingState{$$listOperatingStates{$key}} . " : ";
    #    $all_str .= $label_svSvcUninstalled{$$listUninstalled{$key}} . " ";
        $all_str .= $$listNames{$key}."\n";
    }
    if( !defined($o_descr) ){
        print "UNKNOWN : NO STATUS. Show only\n";
        print "$all_str";
        exit $ERRORS{"UNKNOWN"};
    }
}

my $test=undef;
foreach my $key ( keys %$listNames) {
    $test=undef;
    foreach my $List (@o_descrL) {
        if (!($test)) {
  	        $test = defined($o_noreg) ? $$listNames{$key} eq $List : $$listNames{$key} =~ /$List/i;
        }
    }
    if (!$test) {
        delete $$listNames{$key};
    }
}

##For usual stdout
if( !defined($o_showall) && !defined($o_brief)){
    foreach my $key ( keys %$listNames) {
        $all_str .= $label_svSvcInstalledState{$$listInstallStates{$key}} . " and "; 
        $all_str .= $label_svSvcOperatingState{$$listOperatingStates{$key}} . " : ";
    #    $all_str .= $label_svSvcUninstalled{$$listUninstalled{$key}} . " ";
        $all_str .= $$listNames{$key}."\n";
    }
}

my $nb_found=0;
foreach my $key ( keys %$listNames) {
    $nb_found++;
}
    
my $type;
if( $o_noreg ) {
    $type = 'equal to';
}
else{
    $type = 'with pattern';
}

if( $nb_found == 0 ){
    print "CRITICAL : Found $nb_found $type ($o_descr)\n$all_str";
    exit $ERRORS{"CRITICAL"};
}
elsif( defined($o_number) && $o_number != $nb_found ){
    print "WARNING : Found $nb_found/$o_number $type ($o_descr)\n$all_str";
    exit $ERRORS{"WARNING"};
}
else{
    print "OK : Found $nb_found $type ($o_descr)\n$all_str";
    exit $ERRORS{"OK"};
}
