#!/usr/bin/perl -w
############################## check_snmp_xclarity #################
my $Version='1.0';
# Date : Oct 18 2018
# Author  : Julien George (j.george@esia-sa.com)
# Help : http://wiki.esia-sa.com
#################################################################
#
# Help : ./check_snmp_xclarity.pl -h
#

use strict;
use Switch;
use Net::SNMP;
use Getopt::Long;

# Nagios specific

my $TIMEOUT = 50;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

# SNMP Datas
## For CPU health
my $cpu_base_oid = "1.3.6.1.4.1.19046.11.1.1.5.20.1";
my $cpu_id 			= "1";	#1
my $cpu_desc 		= "2";	#"CPU 1"
#my $cpu_speed 		= "3";	#2100
#my $cpu_identifier = "4";	#"3030 3431 3635 3330 "
#my $cpu_type 		= "5";	#"CENTRAL"
#my $cpu_family 	= "6";	#"Intel Xeon"
my $cpu_cores 		= "7";	#8
my $cpu_threads 	= "8";	#16
#my $cpu_voltage 	= "9";	#1250
my $cpu_datawidth 	= "10";	#64
my $cpu_healthstatus= "11";	#"Normal"
my $cpu_model 		= "12";	#"Intel(R) Xeon(R) Silver 4110 CPU @ 2.10GHz"

# Globals
my $o_host = 	undef; 		# hostname
my $o_community = undef; 	# community
my $o_port = 	161; 		# port
my $o_help=	undef; 		# wan't some help ?
my $o_verb=	undef;		# verbose mode
my $o_version=	undef;		# print version

# End compatibility
my $o_warn=	undef;		# warning level
my @o_warnL=	undef;		# warning levels for Linux Load or Cisco CPU
my $o_crit=	undef;		# critical level
my @o_critL=	undef;		# critical level for Linux Load or Cisco CPU
my $o_timeout=  undef; 		# Timeout (Default 5)
my $o_perf=     undef;          # Output performance data
my $o_version2= undef;          # use snmp v2c
# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password
my $o_ignore= undef;

# functions

sub p_version { print "check_snmp_xclarity : $Version\n"; }

sub print_usage {
    print "Usage: $0 [-v] -H <host> -C <snmp_community> -T [cpu|mem|disk|power|fan|temp|volt] [-2] | (-l login -x passwd [-X pass -L <authp>,<privp>])  [-p <port>] [-i] [-w treshold] [-c treshold] [-t <timeout>] [-V]\n";
}

sub isnnum { # Return true if arg is not a number
  my $num = shift;
  if ( $num =~ /^(\d+\.?\d*)|(^\.\d+)$/ ) { return 0 ;}
  return 1;
}

sub readOID {
        my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub help {
   print "\nSNMP xClarity Monitor\n";
   print_usage();
   print <<EOT;
-v, --verbose
   print extra debugging information
-h, --help
   print this help message
-H, --hostname=HOST
   name or IP address of host to check
-C, --community=COMMUNITY NAME
   community name for the host's SNMP agent (implies v1 protocol)
-2, --v2c
   Use snmp v2c
-l, --login=LOGIN ; -x, --passwd=PASSWD
   Login and auth password for snmpv3 authentication
   If no priv password exists, implies AuthNoPriv
-X, --privpass=PASSWD
   Priv password for snmpv3 (AuthPriv protocol)
-L, --protocols=<authproto>,<privproto>
   <authproto> : Authentication protocol (md5|sha : default md5)
   <privproto> : Priv protocole (des|aes : default des)
-P, --port=PORT
   SNMP port (Default 161)
-T, --type=cpu|mem|disk|power|fan|temp|volt
		cpu   : check CPU status health
		mem   : check Memory DIMM status health
		disk  : check disk status health
		power : check Power status health
		fan   : check fan status health and speed
		temp  : check temp sensors health and values
		volt  : check voltage sensors health and values
-i, --ignore
	to ignore unknown components
-w, --warning=INTEGER[,INTEGER]
	max warning treshold (for type : fan|temp|volt) or min,max warning treshold (type volt only)
-c, --critical=INTEGER[,INTEGER]
	max critical treshold (for type : fan|temp|volt) or min,max critical treshold (type volt only)
-t, --timeout=INTEGER
   timeout for SNMP in seconds (Default: 5)
-V, --version
   prints version number

A LIST is a comma (,) separted list of range. Each range is one of :
  N     N'th byte, character or field, counted from 1
  N-    from N'th byte, character or field, to end of line
  N-M   from N'th to M'th (included) byte, character or field
  -M    from first to M'th (included) byte, character or field
EOT
}

# For verbose output
sub verb { my $t=shift; print $t,"\n" if defined($o_verb) ; }

sub check_options {
    Getopt::Long::Configure ("bundling");
    GetOptions(
		'v'		=> \$o_verb,		'verbose'		=> \$o_verb,
        'h'     => \$o_help,    	'help'        	=> \$o_help,
        'H:s'   => \$o_host,		'hostname:s'	=> \$o_host,
        'p:i'   => \$o_port,   		'port:i'		=> \$o_port,
        'C:s'   => \$o_community,	'community:s'	=> \$o_community,
		'l:s'	=> \$o_login,		'login:s'		=> \$o_login,
		'x:s'	=> \$o_passwd,		'passwd:s'		=> \$o_passwd,
		'X:s'	=> \$o_privpass,	'privpass:s'	=> \$o_privpass,
		'L:s'	=> \$v3protocols,	'protocols:s'	=> \$v3protocols,
        't:i'   => \$o_timeout,     'timeout:i'     => \$o_timeout,
		'i:s'   => \$o_ignore,      'ignore:s'     	=> \$o_ignore,
		'V'		=> \$o_version,		'version'		=> \$o_version,
		'2'     => \$o_version2,    'v2c'           => \$o_version2,
	);

    # Basic checks
	if (defined($o_timeout) && (isnnum($o_timeout) || ($o_timeout < 2) || ($o_timeout > 60)))
	  { print "Timeout must be >1 and <60 !\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	if (!defined($o_timeout)) {$o_timeout=5;}
    if (defined ($o_help) ) { help(); exit $ERRORS{"UNKNOWN"}};
    if (defined($o_version)) { p_version(); exit $ERRORS{"UNKNOWN"}};
    if ( ! defined($o_host) ) # check host and filter
	{ print_usage(); exit $ERRORS{"UNKNOWN"}}
    # check snmp information
    if ( !defined($o_community) && (!defined($o_login)) )
	  { print "Put snmp login info!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	if ((defined($o_login) || defined($o_passwd)) && (defined($o_community) || defined($o_version2)) )
	  { print "Can't mix snmp v1,2c,3 protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	if (defined ($v3protocols)) {
	  if (!defined($o_login)) { print "Put snmp V3 login info with protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	  my @v3proto=split(/,/,$v3protocols);
	  if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
	  if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
	  if ((defined ($v3proto[1])) && (!defined($o_privpass))) {
	    print "Put snmp V3 priv login info with priv protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	}
}

########## MAIN #######

check_options();

# Check gobal timeout if snmp screws up
if (defined($TIMEOUT)) {
  verb("Alarm at $TIMEOUT + 5");
  alarm($TIMEOUT+5);
} else {
  verb("no global timeout defined : $o_timeout + 10");
  alarm ($o_timeout+10);
}

$SIG{'ALRM'} = sub {
 print "No answer from host\n";
 exit $ERRORS{"UNKNOWN"};
};

# Connect to host
my ($session,$error);
if ( defined($o_login) && ! defined($o_passwd))
{
    ($session, $error) = Net::SNMP->session(
      -hostname         => $o_host,
      -version          => '3',
      -username         => $o_login,
      -timeout          => $o_timeout
    );

}
elsif ( defined($o_login) && defined($o_passwd)) {
  # SNMPv3 login
  verb("SNMPv3 login");
    if (!defined ($o_privpass)) {
  verb("SNMPv3 AuthNoPriv login : $o_login, $o_authproto");
    ($session, $error) = Net::SNMP->session(
      -hostname   	=> $o_host,
      -version		=> '3',
      -username		=> $o_login,
      -authpassword	=> $o_passwd,
      -authprotocol	=> $o_authproto,
      -timeout          => $o_timeout
    );
  } else {
    verb("SNMPv3 AuthPriv login : $o_login, $o_authproto, $o_privproto");
    ($session, $error) = Net::SNMP->session(
      -hostname   	=> $o_host,
      -version		=> '3',
      -username		=> $o_login,
      -authpassword	=> $o_passwd,
      -authprotocol	=> $o_authproto,
      -privpassword	=> $o_privpass,
	  -privprotocol => $o_privproto,
      -timeout          => $o_timeout
    );
  }
} else {
	if (defined ($o_version2)) {
		# SNMPv2 Login
		verb("SNMP v2c login");
		  ($session, $error) = Net::SNMP->session(
		 -hostname  => $o_host,
		 -version   => 2,
		 -community => $o_community,
		 -port      => $o_port,
		 -timeout   => $o_timeout
		);
  	} else {
	  # SNMPV1 login
	  verb("SNMP v1 login");
	  ($session, $error) = Net::SNMP->session(
		-hostname  => $o_host,
		-community => $o_community,
		-port      => $o_port,
		-timeout   => $o_timeout
	  );
	}
}
if (!defined($session)) {
   printf("ERROR opening session: %s.\n", $error);
   exit $ERRORS{"UNKNOWN"};
}

sub search_idx{
	my ($table,$base_oid,$idx_col) = @_;
	my @idx;
	foreach my $key ( keys %$table) {
		if ( $key =~ /$base_oid\.$idx_col\./ ) {
			push @idx, $$table{$key};
			verb("OID : $key, Desc : $$table{$key}");
		}
	}
	return sort @idx;
}

sub calcul_op_status{
	my ($result_status) = @_;
	my $my_status = 3;
    
	if ( ($result_status eq "Operational") || ($result_status eq "Optimal") || ($result_status eq "Online" )) {
		$my_status = 0;
	}
    elsif($result_status =~/Hot\sSpare/)
    {
        $my_status = 0;
    }
	else
    {
		$my_status = 2;
	}
	return $my_status;
}

sub calcul_health_status{
	my ($result_status) = @_;
	my $my_status = 3;
	if ( $result_status eq "Normal" ) {
		$my_status = 0;
	}
	elsif ( $result_status eq "Warning" ){
		$my_status = 1;
	}
	else{
		$my_status = 2;
	}
}

sub compare_status{
	my ($status1,$status2) = @_;
	if (( $status1 == 2 ) || ( $status2 == 2 )){
		return 2;
	}
	elsif (( $status1 == 1 ) || ( $status2 == 1 )){
		return 1;
	}
	elsif (( $status1 == 3 ) || ( $status2 == 3 )){
		return 3;
	}
	else{
		return 0;
	}
}

sub calcul_all_healthstatus{
	my ($table,$base_oid,$status_col,@idx_list) = @_;
	my $all_status = 0;
	foreach my $idx (@idx_list){
		my $result_status=$$table{"$base_oid.$status_col.$idx"};
		my $my_status = calcul_healthstatus($result_status);
		$all_status=compare_status($all_status,$my_status);
		verb("idx : $idx, ResultStatus : $result_status, EsiaStatus : $my_status, AllStatus = $all_status");
	}
	return $all_status;
}

sub str_status{
	my ($status) = @_;
	if( $status == 0 ){
		return "OK";
	}
	elsif( $status == 1 ){
		return "WARN";
	}
	elsif( $status == 2 ){
		return "CRIT";
	}
	else{
		return "UNKNOWN";
	}
}

sub volumes_health{
	my ($session,$volumes) = @_;
	my $base_oid = '1.3.6.1.4.1.19046.11.1.1.13.1.6.1';
	my $idx_col = '1';

	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	foreach my $idx (@idx_list){
		my $name = $$full_table{"$base_oid.2.$idx"};
		my $raid = $$full_table{"$base_oid.4.$idx"};
		my $disks= $$full_table{"$base_oid.7.$idx"};
	}
}

sub controller_health{
	my ($session) = @_;
	my $base_oid = '1.3.6.1.4.1.19046.11.1.1.13.1.2.1';
	my $idx_col = '1';

	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	my $all_status = calcul_all_healthstatus($full_table,$base_oid,'24',@idx_list);

	my $output = '';
	foreach my $idx (@idx_list){
		#Controller
		my $descr 		= $$full_table{"$base_oid.2.$idx"};
		my $manufacturer= $$full_table{"$base_oid.6.$idx"};
		my $model		= $$full_table{"$base_oid.11.$idx"};

		my $fw_ver 		= $$full_table{"$base_oid.4.$idx"};
		my $fw_name		= $$full_table{"$base_oid.35.$idx"};

		print "Controller $idx\n";
		print "-> Name : $descr\n";
		print "-> Board : $manufacturer $model\n";
		print "-> Firmware : $fw_name $fw_ver\n";

		#Battery
		my $bat_manu	= $$full_table{"$base_oid.23.$idx"};
		my $bat_model	= $$full_table{"$base_oid.22.$idx"};

		my $bat_dgn_volt= $$full_table{"$base_oid.30.$idx"};
		my $bat_volt	= $$full_table{"$base_oid.31.$idx"};
		my $bat_conso	= $$full_table{"$base_oid.32.$idx"};

		my $bat_cur_cap	= $$full_table{"$base_oid.28.$idx"};
		my $bat_dgn_cap	= $$full_table{"$base_oid.33.$idx"};

		my $bat_temp	= $$full_table{"$base_oid.34.$idx"};

		my $bat_status	= $$full_table{"$base_oid.24.$idx"};
		my $bat_esia_st = calcul_healthstatus($bat_status);
		print "-> Battery : \n";
		print "--> Status $bat_status - $bat_manu $bat_model (Usage : $bat_conso Capacity : $bat_volt/$bat_cur_cap Manufacturer : $bat_dgn_volt/$bat_dgn_cap Temp : $bat_temp)\n";

		volumes_health($session,$$full_table{"$base_oid.37.$idx"});
	}
}

sub check_raid_controller{
	my ($session) = @_;
	my $base_oid = '1.3.6.1.4.1.19046.11.1.1.13.1.2.1';
	my $idx_col = '1';

	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	my $all_status=0;

	foreach my $idx (@idx_list){
		my $descr 		= $$full_table{"$base_oid.2.$idx"};
		my $manufacturer= $$full_table{"$base_oid.6.$idx"};
		my $model		= $$full_table{"$base_oid.11.$idx"};

		my $fw_ver 		= $$full_table{"$base_oid.4.$idx"};
		my $fw_name		= $$full_table{"$base_oid.35.$idx"};

		my $bat_manu	= $$full_table{"$base_oid.23.$idx"};
		my $bat_model	= $$full_table{"$base_oid.22.$idx"};

		my $bat_dgn_volt= $$full_table{"$base_oid.30.$idx"};
		my $bat_volt	= $$full_table{"$base_oid.31.$idx"};
		my $bat_conso	= $$full_table{"$base_oid.32.$idx"};

		my $bat_cur_cap	= $$full_table{"$base_oid.28.$idx"};
		my $bat_dgn_cap	= $$full_table{"$base_oid.33.$idx"};

		my $bat_temp	= $$full_table{"$base_oid.34.$idx"};

		my $bat_status	= $$full_table{"$base_oid.24.$idx"};
		my $bat_esia_st = calcul_op_status($bat_status);
		my $bat_esia_str= str_status($bat_esia_st);
		$all_status = compare_status($all_status,$bat_esia_st);

		print "Raid Ctrl $idx - $descr\n";
		print "=> $manufacturer $model - $fw_ver $fw_name\n\n";
		print "$bat_esia_str - Battery Raid Ctrl $idx - $bat_manu $bat_model - status $bat_status - Temp : $bat_temp\n";
		print "=> Battery days => Days : $bat_cur_cap (Manufacturer : $bat_dgn_cap)\n";
		print "=> Battery load => Usage : $bat_conso - Capacity : $bat_volt (Manufacturer : $bat_dgn_volt)\n\n";
	}
	return $all_status;
}

sub check_raid_drive{
	my ($session) = @_;
	my $base_oid = '1.3.6.1.4.1.19046.11.1.1.13.1.3.1';
	my $idx_col = '1';

	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	my $all_status=0;

	foreach my $idx (@idx_list){
		my $name 		= $$full_table{"$base_oid.2.$idx"};
		my $model		= $$full_table{"$base_oid.3.$idx"};
		my $sn			= $$full_table{"$base_oid.17.$idx"};
		my $slot		= $$full_table{"$base_oid.5.$idx"};

		my $capacity	= $$full_table{"$base_oid.12.$idx"};
		my $speed		= $$full_table{"$base_oid.9.$idx"};
		my $rpm			= $$full_table{"$base_oid.21.$idx"};

		my $temp		= $$full_table{"$base_oid.10.$idx"};

		my $status		= $$full_table{"$base_oid.4.$idx"};
		my $health		= $$full_table{"$base_oid.11.$idx"};

		my $esia_op_stat= calcul_op_status($status);
		my $esia_ht_stat= calcul_health_status($health);
		my $esia_status	= compare_status($esia_op_stat,$esia_ht_stat);

		my $esia_str	= str_status($esia_status);
		$all_status = compare_status($all_status,$esia_status);

		print "$esia_str - $name $model (sn:$sn, slot:$slot, cap:$capacity) status $status, health $health, temp : $temp\n";
	}
	return $all_status;
}

sub check_raid_volume{
	my ($session) = @_;
	my $base_oid = '1.3.6.1.4.1.19046.11.1.1.13.1.7.1';
	my $idx_col = '1';

	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	my $all_status=0;

	foreach my $idx (@idx_list){
		my $name 		= $$full_table{"$base_oid.2.$idx"};
		my $capacity	= $$full_table{"$base_oid.5.$idx"};

		my $status		= $$full_table{"$base_oid.4.$idx"};

		my $esia_status = calcul_op_status($status);
		my $esia_str	= str_status($esia_status);
		$all_status = compare_status($all_status,$esia_status);

		print "$esia_str - $name ($capacity) status $status\n";
	}
	print "\n";
	return $all_status;
}

sub check_raid{
	my ($session) = @_;
	my $all_status=0;
	print "-------------RAID Controllers information-------------\n";
	$all_status=compare_status($all_status,check_raid_controller($session));
	print "-------------Volumes in RAID Controller--------------\n";
	$all_status=compare_status($all_status,check_raid_volume($session));
	print "-------------Drives in RAID Controller---------------\n";
	$all_status=compare_status($all_status,check_raid_drive($session));

	return $all_status;
}

my $i_status=3;
$i_status=check_raid($session);
$session->close;

exit $i_status;
