#!/usr/bin/perl -w 
############################## get_snmp_process.pl #################
my $Version='1.0';
# Date : Feb 26 2015
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use Data::Dumper;

use FindBin;
use lib $FindBin::Bin.'/lib';
use GesaProtocol;

my $PROGNAME='get_snmp_process.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $o_verb=	undef;		# verbose mode
my ($opt_h,$opt_V);
my $community = "public";
my $snmp_version = 2;
my $stat=0;
my ($hostname);
my ($snmp_seclvl,$snmp_user,$snmp_auth_prot,$snmp_auth_key,$snmp_priv_prot,$snmp_priv_key);

my $line;

sub verb { my $t=shift; print $t,"\n" if defined($o_verb) ; }

my $protocol=GesaProtocol->new(4000,"\r\n");

sub print_help() {
	printf "get SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community or v3 auth (defaults to public)\n";
	print "                     (v3 specified as username:authpassword:... )\n";
	print "                       username = SNMPv3 security name\n";
	print "                       authpassword = SNMPv3 authentication pass phrase (or hexidecimal key)\n";
	print "                       authprotocol = SNMPv3 authentication protocol (md5 (default) or sha)\n";
	print "                       privpassword = SNMPv3 privacy pass phrase (or hexidecmal key)\n";
	print "                       privprotocol = SNMPv3 privacy protocol (des (default) or aes)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print "                     3 for SNMP v3\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	'V'	=> \$o_verb,	'verbose'	=> \$o_verb,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"s=i" => \$stat,	"stat=i"=>\$stat,
	"l=i" => \$line,	"line=i"=>\$line,
        #SNMPV3
        "L=s" => \$snmp_seclvl, "snmp_seclvl=s" => \$snmp_seclvl,
        "u=s" => \$snmp_user,   "snmp_user=s" => \$snmp_user,
        "a=s" => \$snmp_auth_prot, "snmp_auth_prot=s" => \$snmp_auth_prot,
        "A=s" => \$snmp_auth_key, "snmp_auth_key=s" => \$snmp_auth_key,
        "x=s" => \$snmp_priv_prot, "snmp_priv_prot=s" => \$snmp_priv_prot,
        "X=s" => \$snmp_priv_key, "snmp_priv_key=s" => \$snmp_priv_key
);

# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }

my ($snmp, $snmperror);
if ($snmp_version == 3)
{
        if(!defined( $snmp_user ) || !defined( $snmp_auth_prot ) || !defined( $snmp_auth_key ) ){
                printf("UNKNOWN ERROR: Missing importants SNMPv3 parameters (user, auth_prot, auth_key)\n");
                exit $ERRORS{"UNKNOWN"};
        }
        if( $snmp_seclvl =~ /authNoPriv/i ){
                ($snmp, $snmperror) = Net::SNMP->session(
                        -hostname => $hostname,
                        -version => 'snmpv3',
                        -username => $snmp_user,
                        -authprotocol => lc $snmp_auth_prot,
                        -authpassword => $snmp_auth_key,
                        -translate   => [ -timeticks => 0x0 ]  # Turn off so sysUpTime is numeric]
                );
        }
        elsif( $snmp_seclvl =~ /authPriv/i ){
                if(!defined( $snmp_priv_prot ) || !defined( $snmp_priv_key ) ){
                        printf("UNKNOWN ERROR: Missing importants authPriv SNMPv3 parameters (priv_prot, priv_key)\n");
                        exit $ERRORS{"UNKNOWN"};
                }
                ($snmp, $snmperror) = Net::SNMP->session(
                        -hostname => $hostname,
                        -version => 'snmpv3',
                        -username => $snmp_user,
                        -authprotocol => lc $snmp_auth_prot,
                        -authpassword => $snmp_auth_key,
                        -privprotocol => lc $snmp_priv_prot,
                        -privpassword => $snmp_priv_key,
                        -translate   => [ -timeticks => 0x0 ]  # Turn off so sysUpTime is numeric]
                );
        }
        else{
                printf("UNKNOWN ERROR: Unknown security level for SNMPv3\n");
                exit $ERRORS{"UNKNOWN"};
        }
}
elsif ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community, 
		-translate   => [ -timeticks => 0x0 ]  # Turn off so sysUpTime is numeric]
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community
		-translate   => [ -timeticks => 0x0 ]
	);
}
if($snmperror ne ""){
        printf($snmperror."\n");
        exit $ERRORS{"UNKNOWN"};
}
# SNMP Datas
my $table='1.3.6.1.2.1.2.2.1';
my %oidmap=( 
	'ifIndex'	=>  "$table.1",
	'ifDescr'	=>  "$table.2",
        'ifType'	=>  "$table.3",
        'ifMtu' 	=>  "$table.4",
        'ifPhysAddress'	=>  "$table.6",
        'ifAdminStatus'	=>  "$table.7",
        'ifOperStatus' 	=>  "$table.8",
	'ifSpeed' 	=>  "$table.5");
	
my @columnNames= qw(
	ifIndex
	ifDescr
	ifType
	ifMtu
	ifPhysAddress
	ifAdminStatus
	ifOperStatus
	ifSpeed);

my $resultat = $snmp->get_entries( -columns => [ map { $oidmap{$_} } @columnNames ] );
#my $resultat = $snmp->get_table(Baseoid => $table);
if (!defined($resultat)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$resultat = $snmp->get_table(Baseoid => $table);
		if (!defined($resultat)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
   		$snmp->close;
  		 exit $ERRORS{"UNKNOWN"};
	}
}

my @arr;
foreach my $key ( keys %$resultat){
	if($key =~ /$table.([0-9]*).([0-9]*)$/){
		my $col=$1;
		my $idx=$2;

		$arr[$idx][$col]= $$resultat{$key};
		my $result = $snmp->get_request(".1.3.6.1.2.1.31.1.1.1.1.".$idx);
		$arr[$idx][9] = $$result{".1.3.6.1.2.1.31.1.1.1.1.".$idx};
	}
}

my $ret;
my $i=1;
my $min=0;
$min=$line if (defined $line);
$protocol->addLine(@columnNames) if($min==0);
foreach my $l (@arr){
	if(defined $l){
		if($i >= $min)
		{
			$ret=$protocol->addLine(@$l[1,2,3,4,6,7,8,5,9]);
			if($ret!=0)
			{
				 last;
			}
		}
		$i++;
	}
}

$protocol->print();
